"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.invoke = void 0;
process.env.FORCE_COLOR = "0";
const nanolru_1 = __importDefault(require("nanolru"));
const path_1 = require("path");
const path_2 = __importDefault(require("path"));
const util_1 = require("util");
const fs_1 = __importDefault(require("fs"));
const stat = (0, util_1.promisify)(fs_1.default.stat);
const cacheParams = { max: 20, maxAge: 60000 };
const caches = {
    configCache: new nanolru_1.default(cacheParams),
    importCache: new nanolru_1.default(cacheParams),
    parentCache: new nanolru_1.default(cacheParams),
};
async function isDir(path) {
    try {
        const fsStat = await stat(path);
        return fsStat.isDirectory();
    }
    catch (e) {
        return false;
    }
}
async function findParent(start, search) {
    const cacheKey = `${start}|${search}`;
    const cachedValue = caches.parentCache.get(cacheKey);
    if (cachedValue === false) {
        return undefined;
    }
    if (cachedValue !== null) {
        return cachedValue;
    }
    const parent = path_2.default.join(start, "..");
    if (parent === start) {
        caches.parentCache.set(cacheKey, false);
        return undefined;
    }
    try {
        const candidate = path_2.default.join(parent, search);
        if (await isDir(candidate)) {
            caches.parentCache.set(cacheKey, candidate);
            return candidate;
        }
    }
    catch (e) { }
    return await findParent(parent, search);
}
async function pluginSearchDirs(cwd) {
    const result = [];
    const localPath = path_2.default.join(cwd, "node_modules");
    if (await isDir(localPath)) {
        result.push(localPath);
    }
    const parentNodeModules = await findParent(__dirname, "node_modules");
    if (parentNodeModules) {
        result.push(parentNodeModules);
    }
    return result;
}
async function resolveConfig(prettier, cwd, filepath) {
    let v = caches.configCache.get(cwd);
    if (!v) {
        v = await prettier.resolveConfig(filepath, {
            editorconfig: true,
            useCache: false,
        });
        if (!v && process.env.PRETTIERD_DEFAULT_CONFIG) {
            console.log(process.env.PRETTIERD_DEFAULT_CONFIG);
            v = await prettier.resolveConfig((0, path_1.dirname)(process.env.PRETTIERD_DEFAULT_CONFIG), {
                config: process.env.PRETTIERD_DEFAULT_CONFIG,
                editorconfig: true,
                useCache: false,
            });
        }
        if (v) {
            caches.configCache.set(cwd, v);
        }
    }
    return {
        ...v,
        filepath,
    };
}
async function resolvePrettier(cwd) {
    const cached = caches.importCache.get(cwd);
    if (cached) {
        return cached;
    }
    return Promise.resolve().then(() => __importStar(require(require.resolve("prettier", { paths: [cwd] })))).catch(() => Promise.resolve().then(() => __importStar(require("prettier"))))
        .then((v) => {
        caches.importCache.set(cwd, v);
        return v;
    });
}
function resolveFile(cwd, fileName) {
    if (path_2.default.isAbsolute(fileName)) {
        return [fileName, fileName];
    }
    return [cwd, path_2.default.join(cwd, fileName)];
}
async function run(cwd, args, text) {
    const fileName = args[0] === "--no-color" ? args[1] : args[0];
    const [cacheKey, fullPath] = resolveFile(cwd, fileName);
    const prettier = await resolvePrettier(cwd);
    const options = await resolveConfig(prettier, cacheKey, fullPath);
    return prettier.format(text, {
        ...options,
        // @ts-ignore: pluginSearchDirs is not declared in the prettier interface :(
        pluginSearchDirs: await pluginSearchDirs(cwd),
    });
}
function invoke(cwd, args, text, _mtime, cb) {
    run(cwd, args, text)
        .then((resp) => void cb(undefined, resp))
        .catch((error) => void cb(error));
}
exports.invoke = invoke;
